@@ -581,19 +581,16 @@ psscan(const char *filename, int scanstyle)
 	    } else {
 		int page_order;
 		
-		switch (sscanf(line+length("%%Pages:"), "%d %d",
+		switch (sscanf(line+length("%%Pages:"), "%u %d",
 			       &maxpages, &page_order)) {
 		    case 2:
 			if (page_order_set == NONE) {
 			    if (page_order == -1) {
 				doc->pageorder = DESCEND;
-				page_order_set = 1;
 			    } else if (page_order == 0) {
 				doc->pageorder = SPECIAL;
-				page_order_set = 1;
 			    } else if (page_order == 1) {
 				doc->pageorder = ASCEND;
-				page_order_set = 1;
 			    }
 			}
 		    case 1:
@@ -864,6 +861,20 @@ psscan(const char *filename, int scanstyle)
 	          iscomment(line+2, "Page:") ||
 	          iscomment(line+2, "Trailer") ||
 	          iscomment(line+2, "EOF")))) {
+	    if (iscomment(line, "%!PS")) {
+	        /* Embedded document in Prolog, typically font resources.
+		 * Skip until end of resource or Prolog
+		 */
+		while (readline(fd, enddoseps, &line, &position, &line_len) &&
+		       !(DSCcomment(line) &&
+			 (iscomment(line+2, "EndProlog") ||
+			  iscomment(line+2, "BeginSetup") ||
+			  iscomment(line+2, "Page:") ||
+			  iscomment(line+2, "Trailer") ||
+			  iscomment(line+2, "EOF")))) {
+		    section_len += line_len;
+		}
+	    }
 	    if (!preread) section_len += line_len;
 	    preread = 0;
 	}
@@ -1000,7 +1011,7 @@ newpage:
             CHECK_MALLOCED(doc->pages);
 	}
 	label = ps_gettext(line+length("%%Page:"), &next_char);
-	if (sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
+	if (sscanf(next_char, "%u", &thispage) != 1) thispage = 0;
 	if (nextpage == 1) {
 	    ignore = thispage != 1;
 	}
@@ -1111,7 +1122,7 @@ continuepage:
 	    /* Do nothing */
 	} else if (iscomment(line+2, "Page:")) {
 	    PS_free(ps_gettext(line+length("%%Page:"), &next_char));
-	    if (sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
+	    if (sscanf(next_char, "%u", &thispage) != 1) thispage = 0;
 	    if (!ignore && thispage == nextpage) {
 		if (doc->numpages > 0) {
 		    doc->pages[doc->numpages-1].end = position;
@@ -1173,8 +1184,15 @@ continuepage:
     }
     section_len += line_len;
     if (DSCcomment(line) && iscomment(line+2, "EOF")) {
-	    readline(fd, enddoseps, &line, &position, &line_len);
+        readline(fd, enddoseps, &line, &position, &line_len);
 	section_len += line_len;
+    } else if (doc->doseps) {
+        /* No EOF, make sure endtrailer <= ps_end */
+        if (position > doc->doseps->ps_begin + doc->doseps->ps_length) {
+	    position = doc->doseps->ps_begin + doc->doseps->ps_length;
+	    section_len = position - doc->begintrailer;
+	    line_len = 0;
+	}
     }
     doc->endtrailer = position;
     doc->lentrailer = section_len - line_len;
@@ -2116,7 +2134,7 @@ pscopydoc(dest_file,src_filename,d,pagelist)
           fputs(comment, dest_file);
           pages_atend = True;
        } else {
-          switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
+          switch (sscanf(comment+length("%%Pages:"), "%*d %u", &i)) {
              case 1:
                 fprintf(dest_file, "%%%%Pages: %d %d\n", pages, i);
                 break;
@@ -2151,7 +2169,7 @@ pscopydoc(dest_file,src_filename,d,pagelist)
          PS_free(comment);
          continue;
       }
-      switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
+      switch (sscanf(comment+length("%%Pages:"), "%*d %u", &i)) {
          case 1:
             fprintf(dest_file, "%%%%Pages: %d %d\n", pages, i);
             break;
